@@ -13,9 +13,10 @@ def test_diabetes_memorize():
     print "%s(Wrong)/%s(Total). The error rate is %f." % (diff, total, diff/float(total))
   assert diff == 0, "Didn't perfectly memorize, got %d wrong" % diff
 
-from helpers import compare_accuracy
+from helpers import compare_accuracy, compare_hybrid_accuracy
 def test_diabetes_accuracy():
   compare_accuracy(x,y)
+  compare_hybrid_accuracy(x,y)
 
 
 if __name__ == "__main__":
